#version 330 core

layout (location = 0) in vec3 position;
layout (location = 1) in vec2 uv;
layout (location = 2) in vec3 normal;
layout (location = 3) in mat4 model;
layout (location = 7) in vec3 tangent;
layout (location = 8) in vec3 bitangent;

out vec3 worldNormal;
out vec4 worldPosition;
out vec2 texCoord;
out vec4 lightSpacePosition[8];
out mat3 tbn;

uniform mat4 view;
uniform mat4 lightView[8];
uniform mat4 projection;
uniform mat4 lightProjection[8];
uniform int numLights;

void main()
{
	mat3 model_inverse_transposed = transpose(inverse(mat3(model)));
	mat4 L;
	L[0] = vec4(0.5,0.0,0.0,0.0);
	L[1] = vec4(0.0,0.5,0.0,0.0);
	L[2] = vec4(0.0,0.0,0.5,0.0);
	L[3] = vec4(0.5,0.5,0.5,1.0);
	for (int i = 0; i < numLights; i++)
	{
		mat4 lightMVP = L * lightProjection[i] * lightView[i] * model;
		lightSpacePosition[i] = lightMVP * vec4(position, 1.0);
	}
	worldPosition = model * vec4(position,0.0);
	worldNormal = normalize(model_inverse_transposed * normal);
	texCoord = uv;
	mat4 mvp = projection*view*model;
	tbn = transpose(mat3(tangent, bitangent, worldNormal));
	gl_Position = mvp * vec4(position,1.0);
}
