#version 330

layout(points) in;
layout(points) out;
layout(max_vertices = 40) out;

in vec3 _position[];
in vec3 _velocity[];
in vec4 _color[];
in float _lifeTime[];
in float _size[];
in int _type[];
in vec4 _colorChange[];

out vec3 position;
out vec3 velocity;
out vec4 color;
out float lifeTime;
out float size;
out int type;
out vec4 colorChange;

uniform vec3 generatorPosition;
uniform vec3 generatorGravity;
uniform vec3 generatorMinVelocity;
uniform vec3 generatorMaxVelocity;
uniform vec4 generatorColor;
uniform float generatorSize;
uniform float generatorMinLifeTime;
uniform float generatorMaxLifeTime;
uniform vec4 generatorColorChange;

uniform int particlesToGenerate;
uniform float time_delta;

uniform vec3 randomSeed;
vec3 localSeed;


float rand()
{
	// Function taken from http://www.mbsoftworks.sk/index.php?page=tutorials&series=1&tutorial=26
    uint n = floatBitsToUint(localSeed.y * 214013.0 + localSeed.x * 2531011.0 + localSeed.z * 141251.0);
    n = n * (n * n * 15731u + 789221u);
    n = (n >> 9u) | 0x3F800000u;
 
    float fRes =  2.0 - uintBitsToFloat(n);
    localSeed = vec3(localSeed.x + 147158.0 * fRes, localSeed.y*fRes  + 415161.0 * fRes, localSeed.z + 324154.0*fRes);
    return fRes;
} 

void main()
{
	localSeed = randomSeed;
	position = _position[0];
	velocity = _velocity[0];
	color = _color[0];
	lifeTime = _lifeTime[0] - time_delta;
	size = _size[0];
	type = _type[0];
	colorChange = _colorChange[0];
	
	if (type == 0)
	{
		EmitVertex();
		EndPrimitive();
		
		vec3 velocityRange = generatorMaxVelocity - generatorMinVelocity;
		float lifeTimeRange = generatorMaxLifeTime - generatorMinLifeTime;
		
		for(int i=0; i<particlesToGenerate; i++)
		{
			position = generatorPosition;
			velocity = generatorMinVelocity + vec3(velocityRange.x*rand(), velocityRange.y*rand(), velocityRange.z*rand());
			color = generatorColor;
			colorChange = generatorColorChange;
			lifeTime = generatorMinLifeTime + lifeTimeRange * rand();
			size = generatorSize;
			type = 1;
			EmitVertex();
			EndPrimitive();
		}
	}
	else if (lifeTime > 0.00)
	{
		position += velocity*time_delta;
		velocity += generatorGravity*time_delta;
		color -= time_delta*colorChange;
		EmitVertex();
		EndPrimitive();
	}
	
}